from pymodbus.client.sync import ModbusSerialClient as ModbusClient
import logging
from ..const import Const


class RS485_Mgr:
    DATA1 = 1
    DATA2 = 2
    DATA3 = 3
    DATA4 = 4

    def __init__(self, port):
        self._port = port
        self._connection = {}
        self._logging = logging.getLogger(Const.LOGGER_NAME)

    def create_new_connection(
            self, method, baudrate, device_id, byte_size, stop_bits, parity):
        client = None

        if parity == 'none':
            parity = 'N'
        elif parity == 'odd':
            parity = 'O'
        elif parity == 'even':
            parity = 'E'

        client = ModbusClient(
            method=method,
            port=self._port,
            stopbits=stop_bits,
            bytesize=byte_size,
            parity=parity,
            baudrate=baudrate,
            timeout=1)

        self._connection[device_id] = client

    def read_func(
            self, client, device_id, reg_addr, func_code, count, endian_isBig):
        tmpVal = 0
        if func_code == 0x01:
            result = client.read_coils(
                address=reg_addr,
                count=count,
                unit=device_id)
            tmpVal = result.bits[0]

        elif func_code == 0x02:
            result = client.read_discrete_inputs(
                address=reg_addr,
                count=count,
                unit=device_id)
            tmpVal = result.bits[0]

        elif func_code == 0x03:
            result = client.read_holding_registers(
                address=reg_addr,
                count=count,
                unit=device_id)
            if count == 1:
                tmpVal = result.registers[0]
            elif count == 2:
                if endian_isBig is True:
                    tmpVal = (result.registers[0] << 16) + result.registers[1]
                else:
                    tmpVal = (result.registers[1] << 16) + result.registers[0]

        elif func_code == 0x04:
            result = client.read_input_registers(
                address=reg_addr,
                count=count,
                unit=device_id)
            if count == 1:
                tmpVal = result.registers[0]
            elif count == 2:
                if endian_isBig is True:
                    tmpVal = (result.registers[0] << 16) + result.registers[1]
                else:
                    tmpVal = (result.registers[1] << 16) + result.registers[0]
        return tmpVal

    def write_func(
            self,
            client, device_id, reg_addr, func_code, count, write_data):
        if func_code == 0x05:
            result = client.write_coil(
                address=reg_addr,
                value=write_data,
                unit=device_id)

        elif func_code == 0x06:
            result = client.write_register(
                addres=reg_addr,
                value=write_data,
                unit=device_id)

        elif func_code == 0x0F:
            result = client.write_coils(
                address=reg_addr,
                value=write_data,
                unit=device_id)

        elif func_code == 0x10:
            result = client.write_registers(
                addres=reg_addr,
                value=write_data,
                unit=device_id)

        return result

    def write_command(
            self, method, baudrate, byte_size, stop_bits, parity,
            device_id, reg_addr, func_code, write_byte):

        if parity == 'none':
            parity = 'N'
        elif parity == 'odd':
            parity = 'O'
        elif parity == 'even':
            parity = 'E'

        client = ModbusClient(
            method=method,
            port=self._port,
            stopbits=stop_bits,
            bytesize=byte_size,
            parity=parity,
            baudrate=baudrate,
            timeout=1)

        try:
            if client.connect():
                if func_code > 0x04:
                    self.write_func(
                        client, device_id, reg_addr,
                        func_code, 1, write_byte)
                client.close()
            else:
                self._logging.error("RS485 client.connect() is error.")
            client.close()
        except Exception as e:
            self._logging.error("RS485 client.connect() is error.")
            self._logging.error(e.args)
            client.close()

    def read(self,
             device_id, reg_addr, func_code, count, endian_isBig):
        if device_id not in self._connection:
            return
        client = self._connection[device_id]

        try:
            if client.connect():
                if func_code <= 0x04:
                    result = self.read_func(
                        client, device_id, reg_addr, func_code,
                        count, endian_isBig)
                    return result
                client.close()
                return result
            else:
                self._logging.error("RS485 client.connect() is error.")
            client.close()
        except Exception as e:
            self._logging.error("RS485 client.connect() is error.")
            self._logging.error(e.args)
            client.close()


#
# EOF
#
