import logging
from ..const import Const
from ..modules.lib.reporter import Reporter
from ..modules.lib.report import Report


class CpuTempReporter(Reporter):
    def __init__(self, start, interval, is_send):
        super().__init__()
        self._prop = {}
        self._prop["CPU_temp_polling_interval"] = interval
        self._prop["CPU_temp_is_send"] = is_send

        self._oneshot = False
        self._oneshot_send = False
        self._start = start
        self._logging = logging.getLogger(Const.LOGGER_NAME)

    def data_type(self):
        return 'temperature'

    def oneshot_report(self):
        self._oneshot = True

    def report(self):
        report = None

        if self._start is False:
            return (report, None)
        if self._oneshot is False or \
                (self._oneshot is True and self._oneshot_send is False):
            temp = None
            try:
                with open('/sys/class/thermal/thermal_zone0/temp') as file:
                    temp = int(file.read()) / float(1000)
            except Exception as e:
                self._logging.error("CPU_temp read error.")
                self._logging.error(e.args)

            report = Report.report_now(
                'measurement',
                CPU_temp=temp
            )
            self._oneshot_send = True
        return (report, None)

#
# End of File
#
